﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

///
///多层神经网络源代码，版权所有 2016 张磊
///有关于此程序代码的详细说明，
///参见 http://www.qiantec.cn/post/wb1
///
namespace NNR
{
    public class NN
    {
        int weishu;
        public OutLevel[] ols;
        NN() { }
        public static NN Load(string file)
        {
            System.IO.FileStream fs = System.IO.File.OpenRead(file);
            System.IO.BinaryReader br = new System.IO.BinaryReader(fs);
            NN n = new NN();
            int nnCount = br.ReadInt32();
            n.weishu = br.ReadInt32();
            n.ols = new OutLevel[nnCount];
            for (int i = 0; i < nnCount; i++)
                n.ols[i] = OutLevel.Load(br);
            br.Close();
            fs.Close();
            return n;
        }
        public double Run(double[] inputs)
        {
            if (inputs.Length != weishu)
                throw new Exception("");
            return ols.Select(x => x.Run(inputs)).Average();
        }
    }
    public class OutLevel
    {
        public double[] w;
        public MidLevel nextLevel;
        public double Run(double[] inputs)
        {
            nextLevel.Run(inputs);
            double sum = 0;
            for (int i = 0; i < nextLevel.o.Length; i++)
                sum += w[i] * nextLevel.o[i];
            return 1 / (1 + Math.Exp(-sum));
        }
        OutLevel() { }
        internal static OutLevel Load(System.IO.BinaryReader br)
        {
            OutLevel ol = new OutLevel();
            int nextlevelcount = br.ReadInt32();
            ol.w = new double[nextlevelcount];
            for (int i = 0; i < nextlevelcount; i++)
                ol.w[i] = br.ReadDouble();
            ol.nextLevel = MidLevel.Load(br);
            return ol;
        }
    }
    public class MidLevel
    {
        public double[] o;
        public double[] thta;
        public double[] w;
        public MidLevel nextLevel;
        internal void Run(double[] inputs)
        {
            if (nextLevel != null)
            {
                nextLevel.Run(inputs);
                int thiscount = o.Length;
                int nextcount = nextLevel.o.Length;
                int id = 0;
                for (int i = 0; i < thiscount; i++)
                {
                    double sum = 0;
                    for (int j = 0; j < nextcount; j++)
                        sum += nextLevel.o[j] * w[id++];
                    sum += thta[i];
                    o[i] = 1 / (1 + Math.Exp(-sum));
                }
            }
            else
                o = inputs;
        }
        MidLevel() { }
        internal static MidLevel Load(System.IO.BinaryReader br)
        {
            MidLevel ml = new MidLevel();
            if (!br.ReadBoolean())
                return ml;
            int benc = br.ReadInt32();
            int nextc = br.ReadInt32();
            ml.o = new double[benc];
            ml.thta = new double[benc];
            ml.w = new double[benc * nextc];
            for (int i = 0; i < benc; i++)
                ml.thta[i] = br.ReadDouble();
            for (int i = 0; i < ml.w.Length; i++)
                ml.w[i] = br.ReadDouble();
            ml.nextLevel = MidLevel.Load(br);
            return ml;
        }
    }
}
